SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
IF OBJECT_ID(N'dbo.usp_ProcessRules') IS NOT NULL
    DROP PROCEDURE dbo.usp_ProcessRules;
GO
-- =============================================
-- Author:		Aaron N. Cutshall
-- Create date: 29-Jul-2017
-- Description:	Process measure rule
-- =============================================
CREATE PROCEDURE [dbo].[usp_ProcessRules]
AS
BEGIN
	DECLARE @RowCnt bigint,
			@LevelCnt int = 0;

	IF OBJECT_ID('tempdb..#tbl_Rule_Tree', 'U') IS NOT NULL DROP TABLE #tbl_Rule_Tree;
	CREATE TABLE #tbl_Rule_Tree (
		TopRuleId INT NOT NULL,
		ParentRuleId INT NULL,
		RuleId INT NOT NULL,
		LogicalId TINYINT NOT NULL, -- Use 1=AND, 2=OR, 3=NOT
		LevelCnt INT NOT NULL
		);

	IF OBJECT_ID('tempdb..#tbl_Rule_Result', 'U') IS NOT NULL DROP TABLE #tbl_Rule_Result;
	CREATE TABLE #tbl_Rule_Result (
		ParentRuleId INT,
		RuleId INT,
		MeasureId INT,
		CustomerId INT,
		LevelCnt INT,
		Result TINYINT
		);

	IF OBJECT_ID('dbo.tbl_MeasureResults', 'U') IS NOT NULL DROP TABLE dbo.tbl_MeasureResults;
	CREATE TABLE dbo.tbl_MeasureResults (
		MeasureId INT,
		CustomerId INT,
		DenRuleResult TINYINT NULL,
		NumRuleResult TINYINT NULL
		);

	INSERT INTO #tbl_Rule_Tree (TopRuleId, RuleId, LogicalId, LevelCnt)
		SELECT r.RuleId, r.RuleId, r.LogicalId, 0
		FROM dbo.tbl_Rule r
		INNER JOIN dbo.tbl_Measure m
			ON r.RuleId in (m.DenRuleId, m.NumRuleId);
	SET @RowCnt = @@ROWCOUNT;

	WHILE @RowCnt > 0 BEGIN
		SET @LevelCnt = @LevelCnt + 1;
		INSERT INTO #tbl_Rule_Tree (TopRuleId, ParentRuleId, RuleId, LogicalId, LevelCnt)
		SELECT rt.TopRuleId, r.ParentRuleId, r.RuleId, r.LogicalId, @LevelCnt
		FROM #tbl_Rule_Tree rt
		INNER JOIN dbo.tbl_Rule r
			ON r.ParentRuleId = rt.RuleId
		WHERE rt.LevelCnt = @LevelCnt-1;
		SET @RowCnt = @@ROWCOUNT;
	END

	INSERT INTO #tbl_Rule_Result(ParentRuleId, RuleId, CustomerId, LevelCnt, Result)
		--
		-- Get assertion results for Population, Denominator, and Numerator (where assertions were created in the numerator)
		--
		SELECT rt.RuleId, NULL, cf.CustomerId, rt.LevelCnt+1, cf.IsTrue
		FROM #tbl_Rule_Tree rt
		INNER JOIN dbo.tbl_RuleFact rf
			ON rf.RuleId = rt.RuleId
		INNER JOIN dbo.tbl_CustFact cf
			ON cf.FactId = rf.FactId
		;
		
	WHILE @LevelCnt > 0 BEGIN
		SET @LevelCnt = @LevelCnt - 1;

		INSERT INTO #tbl_Rule_Result(ParentRuleId, RuleId, CustomerId, LevelCnt, Result)
		SELECT a.ParentRuleId, a.RuleId, a.CustomerId, a.LevelCnt,
				CASE WHEN   ((a.LogicalId = 1) AND (a.RecCount = a.RecTotal)) OR		-- AND
							((a.LogicalId = 2) AND (a.RecTotal > 0)) OR					-- OR
							((a.LogicalId = 3) AND (a.RecTotal = 0)) THEN 1 ELSE 0 END	-- NOT
		FROM (SELECT b.ParentRuleId, b.RuleId, b.LogicalId, b.CustomerId, b.LevelCnt,
					COUNT(b.CustomerId) AS RecCount, SUM(b.Result) as RecTotal
				FROM (SELECT rt.ParentRuleId, rt.RuleId, rt.LogicalId, rr.CustomerId, rt.LevelCnt, rr.Result
						FROM #tbl_Rule_Tree rt
						INNER JOIN #tbl_Rule_Result rr
							ON rr.ParentRuleId = rt.RuleId
							AND rr.LevelCnt = rt.LevelCnt+1
						WHERE rt.LevelCnt = @LevelCnt) AS b
						GROUP BY ParentRuleId, RuleId, LogicalId, CustomerId, LevelCnt
						) AS a;
	END

	INSERT INTO dbo.tbl_MeasureResults(MeasureId, CustomerId, DenRuleResult, NumRuleResult)
	SELECT tmp.MeasureId, tmp.CustomerId, SUM(tmp.DenRuleResult) AS DenRuleResult, SUM(tmp.NumRuleResult) AS NumRuleResult
	FROM (
		SELECT m.MeasureId, rr.CustomerId,
			CASE WHEN rr.RuleId = m.DenRuleId THEN rr.Result ELSE 0 END AS DenRuleResult,
			CASE WHEN rr.RuleId = m.NumRuleId THEN rr.Result ELSE 0 END AS NumRuleResult
		FROM dbo.tbl_Measure m
		INNER JOIN #tbl_Rule_Result rr
			ON rr.RuleId IN (m.DenRuleId, m.NumRuleId)
		) AS tmp
	GROUP BY tmp.MeasureId, tmp.CustomerId;

END